/*
 * Distance.c
 *
 * Created: 04.07.2016 11:07:51
 * Author : bent_even_fossum.fla
 */ 

#include <iotiny817.h>
#include <ssd1306.h>
#include <stdint.h>
#include <intrinsics.h>

#define SYSTEM_CLOCK 20000000ul
#define SYSTEM_PRESCALER 16
#define TIMER_PRESCALER 32
#define TIMER_CLOCK (SYSTEM_CLOCK/SYSTEM_PRESCALER/TIMER_PRESCALER)

__no_init uint16_t current_sound_volocity @0x1E;


#define TEMPERATURE_TABLE_SIZE 13
/* Time of flight to distance measurement is given by t = d*2/v
 * Table below is v/2 in cm/s.
 */
const uint16_t sound_speed_table_conv[TEMPERATURE_TABLE_SIZE] = {
	15779, //315.77 m/s @ -25C
	15947, //318.94 m/s @ -20C
	16104, //322.07 m/s @ -15C
	16259, //325.18 m/s @ -10C
	16413, //328.25 m/s @ -5C
	16565, //331.30 m/s @ 0C
	16716, //334.32 m/s @ 5C
	16865, //337.31 m/s @ 10C
	17014, //340.27 m/s @ 15C
	17161, //343.21 m/s @ 20C
	17307, //346.13 m/s @ 25C
	17451, //349.02 m/s @ 30C
	17594, //351.88 m/s @ 35C
};

///* Thresholds for the different temperatures */
const uint8_t temperature_thresholds[TEMPERATURE_TABLE_SIZE+2] = {
        1023>>2,
	638>>2,
	630>>2,
	619>>2,
	606>>2,
	590>>2,
	570>>2,
	547>>2,
	520>>2,
	491>>2,
	458>>2,
	423>>2,
	388>>2,
	351>>2,
	0,
};

/* Magic numbers */
#define DAC_THRESHOLD 130
#define TIMER_PULSE_TRAIN 0x8
#define TIMER_MASK  0x9C

static void oled_init(void)
{
	ssd1306_init();
        ssd1306_clear();
}

static void oled_write(uint16_t distance)
{
        ssd1306_clear();
        uint8_t i = 0;
        ssd1306_set_column_address(50);
        do {
            ssd1306_write_char(distance%10);
            distance /= 10;
        } while(distance);
        
}



static inline void send_tc_data(uint16_t data)
{
	/* d = t*v/2. t = timer_tick/f_timer 
	 * f_timer = f_system_clock/system_prescaler/timer_prescaler
	 * f_timer = 20MHz/16/32 = 39062
	 * d = timer_tick*sound_speed_table_conv[?]/39062
	 */
	oled_write((uint16_t)(((uint32_t)data*current_sound_volocity)/TIMER_CLOCK));
}

static void seq_init(void)
{
	CCL.SEQCTRL0 = 0x4;
}

static void lut0_init(void)
{
	/* High when AC and TCD is low */
	CCL.TRUTH0 = 0x01;
	CCL.LUTCTRL0B = 0x6<<4 | 0x9;
	CCL.LUTCTRL0A = 0x1 | 0x2<<4;
}

static void lut1_init(void)
{
	VPORTC.DIR |= 1 << 1;
	
	PORTMUX.CTRLA = 0x20;
	
	/* High when TCB is high and TCD is low */
	CCL.TRUTH1 = 0x02;
	CCL.LUTCTRL1B = 0x7 | 0x9<<4;
	CCL.LUTCTRL1A = 0x8 | 0x1;
}

static void ccl_init(void)
{
	/* RS latch is reset when TCD starts new cycle, and set by AC */
	CCL.CTRLA = 0x1;
}

static void carrier_timer_init(void)
{
	/* Should be 40Khz. 20Mhz/16/31 = 40322 */
	TCB0.CTRLB = 0x10 | 0x7;
	TCB0.CC = 0xe1e;
	TCB0.CTRLA = 0x1;
}

static void masking_timer_init(void)
{
	/* Compare B used for mask */
	/* Masking for LUT 0 from count 70 -> 0xfff. 0xba ~= 5ms */
	//TCD0.CMPASET = 0;
	TCD0.CMPACLR = TIMER_MASK;
	TCD0.CMPBCLR = 0xfff;
	/* Mask for LUT 1. Output signal 0x10 ~= 410uS */
	TCD0.CMPBSET = TIMER_PULSE_TRAIN;
	/* Input capture on rising edge. */
	TCD0.EVCTRLA = 0x4 | 0x10 | 0x1;
	/* Use system clock with div32 */
	TCD0.CTRLA = 0x2<<3 | 0x3<<5 | 0x1;
	
	/* Event channel for TCD to trigger capture from LUT0 */
	EVSYS.ASYNCUSER6 = 0x3;
        EVSYS.ASYNCCH0 = 0x1;
}

static void ac_and_dac_init(void)
{
	VREF.CTRLA = 0x2;
	/* Value for echo threshold. Should be a bit below idle (VDD/2) */
	DAC0.DATA = DAC_THRESHOLD;
	DAC0.CTRLA = 0x1;
	
	/* Set up AC */
	AC0.MUXCTRLA = 0x3;
	AC0.CTRLA = 0x1 | 0x2;
}

static void set_system_prescaler(void)
{
	/* Set system clock prescaler to 16 */
        CCP = 0xD8;
	CLKCTRL.MCLKCTRLB = 0x3 << 1 | 0x1;
}

static void temp_init(void)
{
	/* Ref for ADC */
	VREF.CTRLA |= 0x2<<4;
        
        /* Triggered by TCD overflow */
	EVSYS.ASYNCCH1 = 0x4;
	EVSYS.ASYNCUSER1 = 0x4;
        
	/* Turn off input buffer for ADC pin */
	PORTB.PIN4CTRL = 0x4;
	//ADC0.CTRLB = ADC_SAMPLNUM_ACC1_gc;
	ADC0.CTRLC = 0x4;
	ADC0.CTRLE = 0x4;
	ADC0.MUXPOS = 0x9;
	ADC0.EVCTRL = 0x1;

	ADC0.CTRLA = 0x3;
}

int main(void)
{
	set_system_prescaler();
	carrier_timer_init();
	masking_timer_init();
	
	ac_and_dac_init();
	
	seq_init();
	lut0_init();
	lut1_init();
	ccl_init();

	temp_init();

	oled_init();
    while (1) 
    {
		if (TCD0.INTFLAGS & 0x4) {
			send_tc_data(TCD0.CAPTUREA);
			TCD0.INTFLAGS = 0x4;
		} else if (ADC0.INTFLAGS & 0x1) {
			/* Search for temperature value */
                        uint8_t i = 0;
                        while(ADC0.RESL < temperature_thresholds[i+1]) i++;
                        current_sound_volocity = sound_speed_table_conv[i];
			ADC0.WINHT = temperature_thresholds[i];
			ADC0.WINLT = temperature_thresholds[i+1];
			ADC0.INTFLAGS = 0x1;
		}
    }
}

